{*******************************************************}
{                                                       }
{       TInstall Component                              }
{                                                       }
{       Copyright (c) 1995 Bill White                   }
{                                                       }
{*******************************************************}

unit Insttype;

{ This form is the Default Choose File Groups Dialog.  By having this source
code, you can make the dialog look however you want! }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Buttons, InstDec;

type
  TdlgChooseGroups = class(TForm)
    pnlButtons: TPanel;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    lbExcluded: TListBox;
    lbIncluded: TListBox;
    pnlInformation: TPanel;
    Name: TLabel;
    sbInclude: TSpeedButton;
    sbExclude: TSpeedButton;
    Label2: TLabel;
    edName: TEdit;
    edSource: TEdit;
    Label3: TLabel;
    edDestination: TEdit;
    Label1: TLabel;
    Label4: TLabel;
    sbSource: TSpeedButton;
    sbDestination: TSpeedButton;
    Label5: TLabel;
    memDescription: TMemo;
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    btnHelp: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure lbIncludedClick(Sender: TObject);
    procedure lbExcludedClick(Sender: TObject);
    procedure sbExcludeClick(Sender: TObject);
    procedure sbIncludeClick(Sender: TObject);
    procedure sbSourceClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure sbDestinationClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
  private
    { Private declarations }
    OldDir: String;
    FCurrentGroupIndex: Integer; { A "pointer" to the current file group being displayed }
    FFileGroups: TFileGroups; { YOU MUST KEEP THIS.  It is a reference to the
                                structure that keeps information about all the
                                file groups. }
    function GetCurrentGroup: TFileGroup;
    procedure SetCurrentGroupIndex(Value: Integer);
    procedure SetCurrentGroupName(Value: String);
    procedure SetButtons;
  public
    { Public declarations }
    property CurrentGroup: TFileGroup read GetCurrentGroup;
    property CurrentGroupIndex: Integer read FCurrentGroupIndex write SetCurrentGroupIndex;
    property CurrentGroupName: String write SetCurrentGroupName;
    property FileGroups: TFileGroups read FFileGroups write FFileGroups; { YOU MUST KEEP THIS. }
  end;

implementation

uses
  FileCtrl;

{$R *.DFM}

{ Move the "pointer" to the group, specified by Value and also update all
the controls on the form. }
procedure TdlgChooseGroups.SetCurrentGroupIndex(Value: Integer);
begin
  FCurrentGroupIndex := Value;
  edName.Text := CurrentGroup.Name;
  edSource.Text := CurrentGroup.DefaultSourceDir;
  edDestination.Text := CurrentGroup.DefaultDestDir;
  memDescription.Lines.Assign(CurrentGroup.Description);
end;

function TdlgChooseGroups.GetCurrentGroup: TFileGroup;
begin
  Result := FileGroups[CurrentGroupIndex];
end;

{ We will probably only know the name of the group that the user wants to see.
I.e. when the user clicks on an item either in the lbExcluded or lbIncluded
list boxes, the only real information we know is the name of the group.  This
procedure sets the "pointer" to the group with a name, Value. }
procedure TdlgChooseGroups.SetCurrentGroupName(Value: String);
var
  GroupIndex: Integer;
begin
  try
    GroupIndex := FileGroups.IndexOf(Value);
    { Just in case we do not find a group in the FileGroups data structure. }
    if GroupIndex = -1 then raise Exception.Create('Group cannot be found in group list');
    { Actually set the "pointer" }
    CurrentGroupIndex := GroupIndex;
  except
    on E: Exception do MessageDlg(E.Message,mtError,[mbOk],0);
  end;
end;

procedure TdlgChooseGroups.FormShow(Sender: TObject);
begin
  { Get the current working directory }
  GetDir(0,OldDir);
  { Put all the groups that are to be included in the installation into the
    lbIncluded list box }
  FileGroups.DefaultGroups(lbIncluded.Items);
  { Similarly, put all the groups that are to be excluded from the installation
    into the lbExcluded list box }
  FileGroups.NotDefaultGroups(lbExcluded.Items);
  lbExcluded.ItemIndex := -1;
  { Select the first item in the lbIncluded list box }
  lbIncluded.ItemIndex := 0;
  { Move the "pointer" to this group to display information about it }
  CurrentGroupName := lbIncluded.Items[0];
  SetButtons;
end;

procedure TdlgChooseGroups.btnCancelClick(Sender: TObject);
begin
  { Quit out of the installation process }
  raise EQuitOut.Create('');
end;

{ This procedure is called when the user clicks on an item in the lbIncluded
list box.  What we want to do is to display information on that group. }
procedure TdlgChooseGroups.lbIncludedClick(Sender: TObject);
begin
  { De-select any selected item in the Excluded list box }
  lbExcluded.ItemIndex := -1;
  { Move the "pointer" to the group that was clicked on }
  CurrentGroupName := lbIncluded.Items[lbIncluded.ItemIndex];
  SetButtons;
end;

{ This procedure is called when the user clicks on a item in the lbExcluded
list box.  What we want to do is to display information on that group. }
procedure TdlgChooseGroups.lbExcludedClick(Sender: TObject);
begin
  { De-select any selected item in the Included list box }
  lbIncluded.ItemIndex := -1;
  { Move the "pointer" to the group that was clicked on }
  CurrentGroupName := lbExcluded.Items[lbExcluded.ItemIndex];
  SetButtons;
end;

{ This procedure enables or disables buttons on the screen depending on whether
certain conditions are met. }
procedure TdlgChooseGroups.SetButtons;
begin
  { If all the groups are excluded, disable the Exclude SpeedButton }
  sbExclude.Enabled := (lbIncluded.Items.Count > 1) and (lbIncluded.ItemIndex <> -1);
  { If all the groups are included, disable the Include SpeedButton }
  sbInclude.Enabled := (lbExcluded.Items.Count > 0) and (lbExcluded.ItemIndex <> -1);
end;

{ This procedure is called when the user wants to exclude a file group into
from list of groups to be installed on his/her machine.  Note that we are
moving the group from the lbIncluded list box to the lbExcluded list box, and
we are also changing the IsDefault value for that group from True to False. }
procedure TdlgChooseGroups.sbExcludeClick(Sender: TObject);
var
  NewIndex: Integer;
begin
  { Exclude the group from the installation }
  CurrentGroup.IsDefault := False;
  { Get the index of the new file group in the lbExcluded list box }
  NewIndex := lbExcluded.Items.Add(lbIncluded.Items[lbIncluded.ItemIndex]);
  { Get rid of the file group from the list of included groups }
  lbIncluded.Items.Delete(lbIncluded.ItemIndex);
  { We now want to select the file group that has just been excluded }
  lbExcluded.ItemIndex := NewIndex;
  { We want to move our "pointer" to this group }
  CurrentGroupName := lbExcluded.Items[NewIndex];
  SetButtons;
end;

{ This procedure is called when the user wants to include a file group into
the list of groups to be installed on his/her machine.  Note that we are
moving the group from the lbExcluded list box to the lbIncluded list box, and
we are also changing the IsDefault value for that group from False to True. }
procedure TdlgChooseGroups.sbIncludeClick(Sender: TObject);
var
  NewIndex: Integer;
begin
  { Include the group in the installation }
  CurrentGroup.IsDefault := True;
  { Get the index of the new file group in the lbIncluded list box }
  NewIndex := lbIncluded.Items.Add(lbExcluded.Items[lbExcluded.ItemIndex]);
  { Get rid of the file group from the list of excluded groups }
  lbExcluded.Items.Delete(lbExcluded.ItemIndex);
  { We now want to select the file group that has just been included }
  lbIncluded.ItemIndex := NewIndex;
  { We want to move our "pointer" to this group }
  CurrentGroupName := lbIncluded.Items[NewIndex];
  SetButtons;
end;

{ This procedure is called when the user hits the SpeedButton next to
edSource edit control.  Essentially, this edit control determines
where the current group is located when the install program is run. }
procedure TdlgChooseGroups.sbSourceClick(Sender: TObject);
var
  SourceDir: String;
begin
  { What is the current value for the source directory }
  SourceDir := CurrentGroup.DefaultSourceDir;
  { Prompt the user for a new source directory with the standard SelectDirectory procedure }
  if SelectDirectory(SourceDir,[],0) then begin
    edSource.Text := SourceDir;
    { Make sure to update the value for the current file group. }
    CurrentGroup.DefaultSourceDir := SourceDir;
    { The current working directory will probably be changed due to the user moving
      around in dialog, so change it back to the old working directory. }
    ChDir(OldDir);
  end;
end;

procedure TdlgChooseGroups.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  { This is probably not necessary. But in case we are in a different
    working directory from whence we started, go back to the old working
    directory. }
  ChDir(OldDir);
end;

{ This procedure is called when the user hits the SpeedButton next to
edDestination edit control.  Essentially, this edit control determines
where the current group will be copied to on the destination machine. }
procedure TdlgChooseGroups.sbDestinationClick(Sender: TObject);
var
  DestDir: String;
begin
  { What is the current value for the destination directory }
  DestDir := CurrentGroup.DefaultDestDir;
  { Prompt the user for a new destination directory with the standard SelectDirectory procedure }
  if SelectDirectory(DestDir,[sdAllowCreate,sdPerformCreate,sdPrompt],0) then begin
    edDestination.Text := DestDir;
    { Make sure to update the value for the current file group. }
    CurrentGroup.DefaultDestDir := DestDir;
    { The current working directory will probably be changed due to the user moving
      around in dialog, so change it back to the old working directory. }
    ChDir(OldDir);
  end;
end;

procedure TdlgChooseGroups.btnHelpClick(Sender: TObject);
begin
  Application.HelpContext(btnHelp.HelpContext);
end;

end.
